@@ -38,7 +38,37 @@ class AlphaBacktest:
         self.position_volume=defaultdict(int)
         self._is_config=True
         self.config=config
-    def run(self,position_details:pd.DataFrame,open_price:Dict[str,pd.DataFrame],close_price:Dict[str,pd.DataFrame],vwap_price:Dict[str,pd.DataFrame],index_price:Dict[str,float])->List[float]:
+    def buy_stock(self,stock,price,volume):
+        
+        self.position_volume[stock]+=volume
+        self._principal-=volume*price*(1+self._buy_cost_fee)
+
+    def sell_stock(self,stock,price,volume):
+        self._principal+=volume*price*(1-self._sell_cost_fee)
+        if self.position_volume[stock]==0:
+            del self.position_volume[stock]
+    def get_net_value(self,close_df):
+        self._net_value=self._principal
+#             print(short_index_return)
+        for stock in self.position_volume:
+            
+            stock_num=self.position_volume[stock]
+            c_p=close_df[stock]
+
+            self._net_value+=c_p*stock_num     
+        return  self._net_value
+    def rebalance(self,close_df,hold_num):
+        # hold_num=len(stock_list)
+        average_money=self._net_value/hold_num
+        for stock in list(self.position_volume.keys()):
+            c_p=close_df[stock]
+            difference_money=average_money-self.position_volume[stock]*c_p
+            if difference_money>0:
+                self.buy_stock(stock,c_p,difference_money/c_p//100*100)
+            else:
+                self.sell_stock(stock,c_p,(-difference_money/c_p)//100*100)
+                
+    def run(self,position_details:pd.DataFrame,open_price:Dict[str,pd.DataFrame],close_price:Dict[str,pd.DataFrame],vwap_price:Dict[str,pd.DataFrame],index_price:Dict[str,float],close_rebalance=True)->List[float]:
         """
         if just want to use close_price,you can input close_price into vwap_price
         index_price:you should input index close price and index_price ,short volume is same to net_value
@@ -57,10 +87,10 @@ class AlphaBacktest:
                 """
                 卖掉股票
                 """
-                v_p=vwap_df[stock]
+                
                 if stock not in stock_list:
-                    self._principal+=self.position_volume[stock]*v_p*(1-self._sell_cost_fee)
-                    del self.position_volume[stock]
+                    v_p=vwap_df[stock]
+                    self.sell_stock(stock,v_p,self.position_volume[stock])
                 
             for stock in stock_list:
                 """
@@ -69,20 +99,15 @@ class AlphaBacktest:
                 if stock not in self.position_volume:
                     v_p=vwap_df[stock]
                     buy_stock_num=(average_money/v_p)//100*100
-                    self.position_volume[stock]+=buy_stock_num
-                    self._principal-=buy_stock_num*v_p*(1+self._buy_cost_fee)
+                    self.buy_stock(stock,v_p,buy_stock_num)
+
             """
             self._principal是剩余可用资金，在每天的收盘时算净值
             """
-            self._net_value=self._principal
-#             print(short_index_return)
-            for stock in self.position_volume:
-                
-                stock_num=self.position_volume[stock]
-                c_p=close_df[stock]
-
-                self._net_value+=c_p*stock_num
-            
+            self._net_value=self.get_net_value(close_df)
+            if close_rebalance:
+                self.rebalance(close_df,hold_num)
+            self._net_value=self.get_net_value(close_df)
             self._net_value_list.append(self._net_value)
             # date_0=date
         alpha_series=pd.Series(self._net_value_list[1:],index=list(position_details.keys()))
